#!/bin/sh

#
# Common installation script for AVG products for Linux/FreeBSD
# Copyright (c) 2010, AVG Technologies CZ, s.r.o., www.avg.com
#


include()
{
	if [ -f "$1" ]; then
		. "$1"
	else
		echo "File: '$1' was not found. Installation aborted."
		exit 1
	fi
}

print_usage()
{
	echo "Usage: ./install.sh [-h, --help] [-y, --yestoall]"
	echo "OPTIONS:"
	echo "   -h, --help    	print this help"
	echo "   -y, --yestoall	assume \"yes\" and default answer to all questions"
}

process_command_line()
{
	while [ -n "$1" ]; do
		case $1 in
			-h | --help)
				print_usage
				exit 0
				;;
			-y | --yestoall)
				YES_TO_ALL=1
				;;
			*) 
				echo 
				echo "Invalid option: $1" 
				echo
				print_usage
				exit 1
				;;
		esac
		shift
	done
}

set_architecture()
{
	TARGETOS=$(uname -s)
	TARGETMACH=$(uname -m)

	case $TARGETOS in
		Linux)
			STAT_MODE_OPTION="-c %a"
			if [ -n "$(file $SRC_TREE/bin/$AVG_PROG | grep FreeBSD)" ]; then
				abort " ! This package is meant for FreeBSD, and you are trying to install it on Linux." "$ABORT_TEXT"
			fi
			;;
		FreeBSD)
			STAT_MODE_OPTION="-f %Mp%Lp"
			if [ -n "$(file $SRC_TREE/bin/$AVG_PROG | grep Linux)" ]; then
				abort " ! This package is meant for Linux, and you are trying to install it on FreeBSD." "$ABORT_TEXT"
			fi
			;;
		GNU/kFreeBSD)
			STAT_MODE_OPTION="-c %a"
			if [ -n "$(file $SRC_TREE/bin/$AVG_PROG | grep Linux)" ]; then
				abort " ! This package is meant for FreeBSD, and you are trying to install it on Linux." $ABORT_TEXT
			fi
			;;
		*)
			abort "! Other operating system then Linux or FreeBSD, are not yet supported." "$ABORT_TEXT"
			;;
	esac

	case $TARGETMACH in
    	i[3456]86|x86_64|amd64)
			;;
    	*)
			abort "! Other target machine then x86 or x86_64 (amd64) is not yet supported." "$ABORT_TEXT"
			;;
	esac
}

detect_distribution()
{
	local DISTRO="$(detect_distro)"
	local NUMBER=1
	local OLD_IFS="$IFS"

	LINUX_DIST="generic"
	DISTRIBUTION_TEXT="GNU/Linux"
	IFS="
"
	for d in $DISTRIBUTIONS; do
		if [ "${d%:*}" = "$DISTRO" ]; then
			LINUX_DIST="${d%:*}"
			DISTRIBUTION_TEXT="${d#*:}"
			break
		fi
		NUMBER=$(($NUMBER + 1))
	done

	ask_Yorn_question "Operating system is $DISTRIBUTION_TEXT ($TARGETMACH)"
	if [ $? -eq 0 ]; then
		ask_option_question "Select one of these distributions" NUMBER $(echo "$DISTRIBUTIONS" | awk -F: "{print \$2}")
		local d=$(echo "$DISTRIBUTIONS" | awk "{if (NR==$NUMBER) print \$0}")
		LINUX_DIST="${d%:*}"
		DISTRIBUTION_TEXT="${d#*:}"
	fi
	IFS="$OLD_IFS"
}

set_updaterc_d()
{
	detect_distribution
	INIT_FILE_PATH=$DST_INITD_FOLDER/avgd.all
	INIT_SLINK_PATH=/etc/init.d/$AVG_DAEMON
	REGISTER_CMD="/sbin/chkconfig --add $AVG_DAEMON"
	UNREGISTER_CMD="/sbin/chkconfig --del $AVG_DAEMON"

	case "$LINUX_DIST" in
		rh|mdk|suse)
			;;
		deb|ubuntu)
			REGISTER_CMD="update-rc.d $AVG_DAEMON defaults"
			UNREGISTER_CMD="update-rc.d -f $AVG_DAEMON remove"
			;;
		gentoo)
			INIT_FILE_PATH=$DST_INITD_FOLDER/avgd.gentoo
			REGISTER_CMD="rc-update add $AVG_DAEMON default"
			UNREGISTER_CMD="rc-update del $AVG_DAEMON"
			;;
		slack)
			INIT_SLINK_PATH=/etc/rc.d/rc.$AVG_DAEMON
			REG_LEVELS="$(awk '$1 == "#" && $2 == "chkconfig:" {print $3":"$4":"$5}' init.d/avgd.all)"
			REG_LEVELS=${REG_LEVELS:-"2345:60:40"}
			REGISTER_CMD="register_slack_initscript rc.$AVG_DAEMON $REG_LEVELS"
			UNREGISTER_CMD="unregister_slack_initscript rc.$AVG_DAEMON"
			;;
		fbsd)
			INIT_SLINK_PATH=/usr/local/etc/rc.d/${AVG_DAEMON}.sh
			DST_SYMLINK_FOLDERS=$DST_SYMLINK_FOLDERS_FreeBSD
			REGISTER_CMD=
			UNREGISTER_CMD=
			[ "$TARGETMACH" = "amd64" ] && WRAPPER_COMMAND="export LD_32_LIBRARY_PATH=\${LD_LIBRARY_PATH}"
			;;
		*)
			INIT_SLINK_PATH=
			REGISTER_CMD="echo \"Please register the '$AVG_DAEMON' service initscript for startup and runlevels.\""
			UNREGISTER_CMD="echo \"Please unregister the '$AVG_DAEMON' service initscript from startup and runlevels.\""
			;;
	esac

	if [ "$TARGETOS" = "GNU/kFreeBSD" ]
	then
		DST_SYMLINK_FOLDERS=$DST_SYMLINK_FOLDERS_FreeBSD
	fi
}

#
# print head text, EULA
#
head_text_eula()
{
	local HEAD_TEXT="   $PROJECT_NAME_FULL installation   "

	clear
	echo "$HEAD_TEXT" | sed 's/./=/g'
	echo "$HEAD_TEXT"
	echo "$HEAD_TEXT" | sed 's/./=/g'
	if [ -n "$VER_PREVIOUS" ] ; then
		echo "Already installed version:  $(ralign 27)$VER_PREVIOUS"
		echo "You are installing version: $(ralign 27)$VER_CURRENT"
	fi
	ask_yesorno_question "Do you want to continue?" && abort 

	if [ $YES_TO_ALL -eq 0 ] ; then 
		less $LICENSE_TEXT_FILE
		ask_yesorno_question "Do you agree with the license?" && abort "$ABORT_TEXT"
		echo
	fi
}

backup_prior_version()
{
	local BACKUP_FILE="${DST_TREE##*/}_$(date +'%Y%m%d_%H%M').tar.gz"
	
	if [ -f "$DST_TREE/bin/$AVG_PROG" ]; then
		echo "Archiving prior installation... $(ralign 31)${DST_TREE%/*}/$BACKUP_FILE"
		[ -x "$(which tar)" ] || abort "Tar not found - can't archive prior installation." "$ABORT_TEXT"
		(cd ${DST_TREE%/*}; tar -zcf $BACKUP_FILE ${DST_TREE##*/})
	fi
}

create_group_user()
{
	ask_default_answer_question "Under which user should $PROJECT_NAME_SHORT run" USER
	ask_default_answer_question "What group should be used for $PROJECT_NAME_SHORT" GROUP
	
	if [ -z "$(grep "^$GROUP\>" /etc/group | cut -f1 -d:)" ]; then
		case "$LINUX_DIST" in
			fbsd )
				pw groupadd $GROUP
				;;
			*)
				groupadd $GROUP
				;;
		esac
	fi

	if [ -z "$(grep "^$USER\>" /etc/passwd | cut -f1 -d:)" ]; then
		case "$LINUX_DIST" in
			fbsd )
				pw useradd -n $USER -g $GROUP -s ""
				;;
			*)
				useradd -r -s /sbin/nologin -g $GROUP $USER
				;;
		esac
	fi
}

#
# create first part of uninstall script, list of (installed) targets to remove will fallow
#
begin_uninstall_script()
{
	cat <<EOF >$UNINSTALL_TMP
#!/bin/sh
#
# Copyright (c) 2010, AVG Technologies CZ, s.r.o., www.avg.com
# Uninstall script for the $PROJECT_NAME_FULL
#

INIT_SLINK_PATH=$INIT_SLINK_PATH
YES_TO_ALL=$YES_TO_ALL

EOF

	if [ $LINUX_DIST = "slack" ] ; then
		 cat "$SRC_TREE/etc/init.d/functions.slack" >> $UNINSTALL_TMP
	fi
	cat "$SRC_TREE/etc/init.d/functions.setup" "script/$UNINSTALL_FILE" >> $UNINSTALL_TMP

	# generate unregister rule uninstall script
	if [ -n "$UNREGISTER_CMD" ] ; then
		 echo "echo \"Unregistering '$AVG_DAEMON' service ...\"" >> $UNINSTALL_TMP
		 echo $UNREGISTER_CMD >> $UNINSTALL_TMP
	fi

	# generate removing of initscript symlink for uninstall script
	echo "echo \"Uninstalling '$AVG_DAEMON' service initscripts...\"" >> $UNINSTALL_TMP
	if [ -n "$INIT_SLINK_PATH" ] ; then 
		 echo "rm -f $INIT_SLINK_PATH" >> $UNINSTALL_TMP
	else
		 echo "echo \"Automatic uninstallation of initscripts for your platform/distribution is not supported.\"" >> $UNINSTALL_TMP
		 echo "echo \"Please remove the symbolic link to file $INIT_FILE_PATH created manually after installation.\"" >> $UNINSTALL_TMP
	fi
}

create_dirs()
{
	echo "Creating destination directories..."

	if [ ! -d "/opt/avg" ]
	then
		ask_Yorn_question "Directory /opt/avg doesn't exist. Create?"
		if [ $? -eq 0 ]
		then
			abort "Cannot continue in installation!"
		fi
	fi

	for d in $(find $DST_INSTALL_FOLDERS -type d); do

		# create directory (if not exist)
		if [ ! -d "/$d" ]; then
			local MODE="$(stat $STAT_MODE_OPTION $d)"
			mkdir -m $MODE -p /$d 2>/dev/null || abort "Cannot create directory '/$d'!" "$ABORT_TEXT"
		fi

		chown $USER:$GROUP /$d

		# don't uninstall persisting directories
		for f in $BCK_PRESERVED_FILES ; do
			echo "$f" | grep "^/$d" >/dev/null && d=""
		done
		[ -n "$d" ] && UNINSTALL_DIRS="/$d $UNINSTALL_DIRS"
	done
}

install_files()
{
	echo "Installing files..."

	for f in $(find $DST_INSTALL_FOLDERS -type f); do
		local MODE="$(stat $STAT_MODE_OPTION $f)"
		install -m $MODE -o $USER -g $GROUP $f /$f

		# don't uninstall excluded files
		for excluded in $UNINSTALL_EXCLUDE_FILES ; do
			[ "$f" = "$excluded" ] && continue 2
		done
		echo "remove_file '/$f'" >> $UNINSTALL_TMP
	done
}

create_symlinks()
{
	echo "Creating symlinks..."

	for l in $(find $DST_SYMLINK_FOLDERS -type l); do
		[ -h /$l ] && rm /$l
		cp -p -P -R $l /$l
		chown -h $USER:$GROUP /$l
		echo "remove_file '/$l'" >> $UNINSTALL_TMP
	done

	# create initscript symlink
	echo "Installing '$AVG_DAEMON' service initscripts..."
	if [ -n "$INIT_SLINK_PATH" ] ; then
		[ -h "$INIT_SLINK_PATH" ] && rm $INIT_SLINK_PATH
		[ -f /etc/sabayon-release ] && LINK_OPTION=-f || LINK_OPTION=-s
		ln $LINK_OPTION $INIT_FILE_PATH $INIT_SLINK_PATH
	else
		 echo "Automatic installation of initscripts for your platform/distribution is not supported."
		 echo "Please, in directory according to your platform/distribution,"
		 echo "create symbolic link '$AVG_DAEMON' to initscript for AVG daemon"
		 echo "$DST_INITD_FOLDER/avgd.all."
	fi

	if [ -n "$REGISTER_CMD" ] ; then
		echo "Registering '$AVG_DAEMON' service to runlevels..."
		$REGISTER_CMD || echo " warning: Registration to runlevels failed!"
	fi
}

#
# check license and register if needed
#
check_license()
{
	$LICINFO_CMD > /dev/null
	local RESULT=$?
	if [ $RESULT -ne 0 ] ; then

		if [ -n "$LICENSE_NUMBER" ] ; then
			echo "Registering with license: $LICENSE_NUMBER"
			$LICREG_CMD "$LICENSE_NUMBER"
			RESULT=$?
		elif [ $YES_TO_ALL -eq 0 ] ; then
			echo "Enter your license number (or press ENTER for later registration):"
			read LICENSE
			if [ -n "$LICENSE" ] ; then
				$LICREG_CMD "$LICENSE"
				RESULT=$?
			fi
		fi

		[ $RESULT -ne 0 ] && RESULT_TEXT="Please register $PROJECT_NAME_FULL with command: $LICREG_CMD <licno>"
	fi
	return $RESULT
}


########
# START
#

cd ${0%/*}
SRC_TREE="$(echo opt/avg/*)"

# include script files and init scripts configuration files
include "$SRC_TREE/etc/init.d/avgdinit.conf"
include "$SRC_TREE/etc/init.d/functions.common"
include "$SRC_TREE/etc/init.d/functions.setup"
include "$SRC_TREE/etc/init.d/functions.slack"

# define paths for files and directories
LICENSE_TEXT_FILE="$SRC_TREE/doc/license_us.txt"
UNINSTALL_TMP="uninstall_tmp.sh"
UNINSTALL_FILE="uninstall.sh"
UNINSTALL_DIRS=""

ABORT_TEXT="Installation aborted."

# default values
YES_TO_ALL=0
RESULT=0

# overwrite product specific functions and settings
include "script/functions.install"

process_command_line $@

pre_install
set_architecture
set_updaterc_d

create_group_user
begin_uninstall_script
create_dirs
install_files
create_symlinks
finish_uninstall_script

post_install
