#!/usr/bin/env bash

#
# Diagnostic script for AVG for Linux/FreeBSD/MacOSX
# Copyright (c) 2010, AVG Technologies, s.r.o., www.avg.com
#

NAME=${0##*/}
PKG_NAME=${NAME%.sh}
AVGINSTDIR=${0%/*}/..
. ${AVGINSTDIR}/etc/init.d/functions.diag

abort()
{
	local REASON="$1"

	if [ ! -z "$REASON" ]; then
		echo
		echo "$REASON"
		echo
	else
		echo "Diagnostic script aborted."
	fi
	exit 1
}

create_dir()
{
	for ARG in "$@"
	do
		mkdir $ARG || abort "Cannot create folder \"$ARG\" for results. Please check your write permissions for current folder."
	done
}

#
# print decorated log message
#
print_log_message()
{
	local LOG_MSG="$1"
	local LOGFILE="$2"

	echo "  $LOG_MSG  " | sed 's/./#/g' >> $LOGFILE
	echo "# $LOG_MSG #" | awk '{ print toupper($0) }' >> "$LOGFILE"
	echo "  $LOG_MSG  " | sed 's/./#/g' >> $LOGFILE
}

#
# do command, print message + log message
#
action()
{
	local CMD="$1"
	local MSG="$2"
	local MSG_DELIMITER=", "

	if [ -n "$3" ]; then
		LOGFILE="$3"
		MSG_DELIMITER=""
	fi

	# message, log
	if [ -n "$MSG" ]; then
		echo -n "$MSG_DELIMITER$MSG"
		print_log_message "$MSG" "$LOGFILE"
	fi
	
	# command
	echo >> "$LOGFILE"
	eval "$CMD >> $LOGFILE 2>&1"
	RETVAL=$?
	echo >> "$LOGFILE"
	return $RETVAL
}

#
# running processes, loaded modules, listened ports
#
get_running_processes()
{
	local LOGFILE="$1"
	
	echo -n "Exporting lists of: "
	action "ps aux"           "running processes" "$LOGFILE"
	action "$get_modules_cmd" "loaded modules"
	action "$get_ports_cmd"   "used ports"
	echo
}

#
# OS information
#
get_system_information()
{
	local LOGFILE="$1"

	echo -n "Exporting system information: "
	action "uname -a"                            "kernel" "$LOGFILE"
	action "$get_lsb_cmd"                        "OS" || action "cat /etc/*-release"
	action "file ${AVGINSTDIR}/bin/${BIN_INFO_FILE_NAME}"
	action "$get_cpuinfo_cmd"                    "CPU"
	action "$get_meminfo_cmd"                    "memory"
	action "sysctl -a 2>/dev/null | grep shmmax" "shared memory"
	action "ipcs -a"                             
	action "$get_env_cmd"                        "environment variables"

	if [ "uname -s" = "Darwin" ];
	then
		echo -n "MacOSX specific information: "
		action sw_vers "MacOSX sw_vers info" "$LOGFILE"
		action system_profiler "MacOSX system_profiler info" "$LOGFILE"
	fi

	echo
}

#
# do core dumps of AVG running processes (do not crush them)
#
make_avg_coredumps()
{
	local DUMPDIR="$1"
	local DUMPINIFILE="$AVGINSTDIR/cfg/dump.ini"

	echo -n "Creating of core dumps of running AVG components: "
	create_dir "$DUMPDIR"

	mv $DUMPINIFILE ${DUMPINIFILE}_temp
	cat <<EOF > ${AVGINSTDIR}/cfg/dump.ini
[default]
actions = GDB_DUMP
EOF

	for PID in `ps axopid,comm | grep avg | grep -v $NAME | awk '{print $1}'`
	do
		local BIN="`ps ocomm= -p $PID`"
		echo -n "$BIN "
		sudo avgdump -p $PID -e ${AVGINSTDIR}/bin/$BIN -a $arch >/dev/null
		mv ${AVGINSTDIR}/log/${BIN}_${PID}.* "$DUMPDIR"
		mv ${AVGINSTDIR}/var/run/${BIN}_${PID}.core "$DUMPDIR"
	done

	mv ${DUMPINIFILE}_temp $DUMPINIFILE
	echo
}

#
# pack collected information
#
create_diag_package()
{
	if [ -x "`which tar`" ]; then
		echo "Packing all data into archive..."
		tar czf "$folder.tar.gz" "$folder"
		if [ "$?" -eq 0 ]; then 
			rm -Rf "$folder"
			echo
			echo "Diagnostic data packed to $folder.tar.gz."
		else
			rm -f "$folder.tar.gz"
			echo
			echo "Problem during compression, please use result in folder $folder."
		fi
	else
		echo "Tar not found - can't pack results, please use results from folder $folder."	
	fi
}



########
# START
#

db=0
dump=0
process_action_line $@

#
# check privilegs
#
if [ $(id -u) -ne 0 ]; then
	echo "It is needed to launch this script as root."
	echo
	exit 1
fi

#
# set variables
#
arch="32"
case "`uname -m`" in
	x86_64|amd64)	arch="64";;
esac
case "`uname -s`" in
	"FreeBSD")
		get_lsb_cmd="echo"
		get_env_cmd="env"
		get_ports_cmd="netstat -ntaptcp"
		get_modules_cmd="kldstat"
		get_cpuinfo_cmd="sysctl hw.model"
		get_meminfo_cmd="sysctl -a 2>/dev/null | grep 'mem'"
		get_binlinks_cmd="ls -l /usr/local/bin/avg*"
	;;
	"Darwin")
		get_lsb_cmd="echo"
		get_env_cmd="env"
		get_ports_cmd="netstat -ntaptcp"
		get_cpuinfo_cmd="sysctl hw.model"
		get_meminfo_cmd="sysctl -a 2>/dev/null | grep 'mem'"
		get_binlinks_cmd="ls -l /usr/local/bin/avg*"
	;;
	*)
		get_lsb_cmd="lsb_release -a"
		get_env_cmd="export -p"
		get_ports_cmd="netstat -ntap"
		get_modules_cmd="cat /proc/modules"
		get_cpuinfo_cmd="cat /proc/cpuinfo"
		get_meminfo_cmd="cat /proc/meminfo"
		get_binlinks_cmd="ls -l /usr/bin/avg*"
	;;
esac

folder=`date +${PKG_NAME}_%Y-%m-%d_%H-%M-%S`

#
# do some work
#
create_dir "$folder"

get_avg_product_information "$folder/info.txt"
get_system_information "$folder/info.txt"
get_running_processes "$folder/processes.txt"
get_avg_configuration "$folder/config.txt"
get_avg_files_information "$folder/avgfiles.txt"
[ "$dump" -eq 1 ] && make_avg_coredumps "$folder/core"
get_logs_cfg_files

create_diag_package
