/#!/bin/sh

#
# Setup script for AVG Admin Server for Linux
# Copyright (c) 2012, AVG Technologies CZ, s.r.o., www.avg.com
#

include()
{
	if [ -f "$1" ]; then
		. "$1"
	else
		echo "File: '$1' not found. Setup aborted."
		exit 1
	fi
}

print_usage()
{
	echo "Usage: ./setup.sh [-h, --help] [-y, --yestoall]"
	echo "   OPTIONS:"
	echo "   -h, --help	print this help"
	echo "   -y, --yestoall	assume "yes", default answer to all questions"
}

process_command_line()
{
	while [ -n "$1" ]; do
		case $1 in
			-h | --help)
				print_usage
				exit 0
				;;
			-y | --yestoall)
				YES_TO_ALL=1
				;;
			*) 
				echo 
				echo "Invalid option: $1" 
				echo
				print_usage
				exit 1
				;;
		esac
		shift
	done
}

detect_db_settings()
{
	echo "Detecting database engine..."

	if [ "$DB_TYPE" = "Firebird" ]; then
		local FB_PID="$(fuser gds_db/tcp 2>/dev/null)"
		local FB_NETSTAT="$(netstat -lntp | grep fbserver)"
		DB_SERVER_PATH="/opt/firebird"

		if [ -n "$FB_PID" ] ; then
			[ -n "$(ps p $FB_PID o comm= | grep inetd)" ] && DB_SERVER_TYPE="CS" || DB_SERVER_TYPE="SS"
		elif [ -n "$FB_NETSTAT" ] ; then
			DB_SERVER_TYPE="SS"
			FB_PID="$(echo "$FB_NETSTAT" | grep fbserver | awk '{split($7,a,"/"); print a[1]}')"
		fi

		case "$DB_SERVER_TYPE" in
			CS)
				read_from_confi "/etc/xinetd.d/firebird" DB_SERVER_FILE "server" "/opt/firebird/bin/fb_inet_server"
				read_from_confi "/etc/xinetd.d/firebird" DB_SERVER_USER "user" "firebird"
				DB_SERVER_PORT="$(awk '$1=="gds_db" {split($2,a,"[/]"); print a[1]; exit 0}' /etc/services 2>/dev/null || echo 3050)"
				DB_SERVER_PATH="${DB_SERVER_FILE%/bin/*}"
				;;
			SS)
				DB_SERVER_FILE="$(ps -p $FB_PID -o cmd=)"
				DB_SERVER_USER="$(ps -p $FB_PID -o user=)"
				DB_SERVER_PORT="$(echo "$FB_NETSTAT" | awk '{split($4,a,":"); print a[2]}')"
				DB_SERVER_PATH="${DB_SERVER_FILE%/bin/*}"
				;;
		esac

		[ -x "$DB_SERVER_FILE" ] && \
			DB_SERVER_VERSION="$($DB_SERVER_FILE -z | awk '{print $7}')"

		# try to get firebird default SYSDBA pw in case instalation from rpm
		read_from_confi "$DB_SERVER_PATH/SYSDBA.password" DBA_USER_PW "ISC_PASSWD" "$DBA_USER_PW"
		[ -f ${DB_SERVER_PATH}/firebird.log  ] && write_to_confi "$DIAG_FILE_PATH" "DB_SERVER_LOG_FILE" "${DB_SERVER_PATH}/firebird.log"
		[ -f ${DB_SERVER_PATH}/firebird.conf ] && write_to_confi "$DIAG_FILE_PATH" "DB_SERVER_CFG_FILE" "${DB_SERVER_PATH}/firebird.conf"
	fi

	if [ -n "$DB_SERVER_TYPE" ]; then
		DB_SERVER_STATUS="(is running)"
	else
		echo
		echo " WARNING: Database server settings not detected!" 
		echo "          Is the $DB_TYPE server installed and running? (see INSTALL,FAQ)"
		echo "          Used Firebird default configuration."
		DB_SERVER_USER="firebird"
		DB_SERVER_PORT="3050"
		DB_SERVER_STATUS="(NOT DETECTED!)"
	fi

	# Test of the minimum supported version of database engine
	local DB_SERVER_VERSION_MAJOR="$(echo $DB_SERVER_VERSION | awk -F. '{print $1}')"
	if [ -n "$DB_SERVER_VERSION_MAJOR" ]; then 
		if [ "$DB_SERVER_VERSION_MAJOR" -lt "$((${DB_TYPE}_SERVER_VERSION_MIN))" ]; then
			echo
			echo " WARNING: Firebird server $DB_SERVER_VERSION is too old!"
			echo "          At least $((${DB_TYPE}_SERVER_VERSION_MIN)).$((${DB_TYPE}_SERVER_VERSION_MIN_MINOR)) is required (see INSTALL)."
		fi
	fi
}

#
# Select database engine, confirm detected settings
#
setup_db_engine()
{
	while [ 1 ]; do
		detect_db_settings
		echo "Database engine: $(ralign 16)$DB_TYPE $DB_SERVER_TYPE $DB_SERVER_VERSION $DB_SERVER_STATUS"
		if [ "$DB_SERVER_STATUS" != "(is running)" ]; then
			YES_TO_ALL=0
			ask_Yorn_question "Try again?                                       " || continue
		fi
		break
	done

	while [ 1 ]; do
		ask_default_answer_question "Which user is the $DB_TYPE server running under?" DB_SERVER_USER
		grep "^$DB_SERVER_USER\>" /etc/passwd > /dev/null
		if [ $? -eq 1 ]; then
			echo " WARNING: Database engine user '$DB_SERVER_USER' not found!"
			YES_TO_ALL=0
			ask_Yorn_question "Try another user?" || continue
		fi
		break
	done
	chown $DB_SERVER_USER $DST_TREE/var/db

	ask_default_answer_question "Which port is the $DB_TYPE server running under?" DB_SERVER_PORT
	if [ "$DB_SERVER_PORT" = "$((${DB_TYPE}_SERVER_PORT_DEFAULT))" ]; then
		CONNECTION_STRING="$DBA_FILE_PATH"
	else
		CONNECTION_STRING="localhost/$DB_SERVER_PORT:$DBA_FILE_PATH"
	fi
	write_to_confi "$CFG_FILE_PATH" "ConnectionString" "$CONNECTION_STRING"

	while [ 1 ]; do
		ask_default_answer_question "Which database owner should be the Admins database created under?" DBA_USER_NAME
		ask_default_answer_question "Database owner password" DBA_USER_PW
		if [ "$DB_TYPE" = "Firebird" ]; then
			local GSEC_PATH="$DB_SERVER_PATH/bin/gsec"
			[ -x "$GSEC_PATH" ] || GSEC_PATH="/usr/bin/gsec"

			if [ ! -x "$GSEC_PATH" ]; then
				DBA_USER_NAME=SYSDBA
				DBA_USER_PW=masterkey
				echo " WARNING: $GSEC_PATH not found!"
				echo "          Default database owner used ($DBA_USER_NAME/$DBA_USER_PW)"
			else
				$GSEC_PATH -user $DBA_USER_NAME -password $DBA_USER_PW -display > /dev/null
				if [ $? -ne 0 ]; then
					echo " WARNING: Database owner/password incorrect!"
					echo " NOTE: You can create a new database owner with the following command:"
					echo "       $GSEC_PATH -user SYSDBA -password <SYSDBA_PASSWORD> -add <OWNER> -pw <PASSWORD>"
					YES_TO_ALL=0
					ask_Yorn_question "Try another database owner/password?" || continue
				fi
			fi
		fi
		break
	done
	write_to_confi "$CFG_FILE_PATH" "DBAUserName" "$DBA_USER_NAME"
	write_to_confi "$CFG_FILE_PATH" "DBAPassword" "$DBA_USER_PW"
}

#
# Configure datacenter
#
setup_dc()
{
	local SELECTED_DB_ACTION=1

	if [ -f "$DBA_FILE_PATH" ]; then

		local OLD_DB_VERSION="$(avgadmsrv --versionDB || echo 'UNKNOWN!')"
		echo "Old database version: $(ralign 21)${OLD_DB_VERSION}"
		echo "New database version: $(ralign 21)${NEW_DB_VERSION:-UNKNOWN!}"

		if [ "$OLD_DB_VERSION" = "$NEW_DB_VERSION" ]; then 
			SELECTED_DB_ACTION=3
			local COND_OPTION="keep old database"
		else
			SELECTED_DB_ACTION=2
		fi
		ask_option_question "Choose database action" SELECTED_DB_ACTION "create new database" "upgrade old database" "$COND_OPTION"
	fi

	case "$SELECTED_DB_ACTION" in
		1)	
			echo "Creating new database...";
			SELECTED_DB_ACTION_TEXT="create database"
			avgadmsrv --createDB && echo "Databse successfully created."
			;;
		2)	
			echo "Upgrading database...";    
			SELECTED_DB_ACTION_TEXT="upgrade database"
			avgadmsrv --executeSQLscript ${DST_TREE}/var/db/scripts/FireBird_Scripts/AVG_ServersDomainFix.sql
			avgadmsrv --upgradeDB && echo "Databse successfully upgraded."
			;;
		*) 
			echo "Keeping old database..."
			;;
	esac

	if [ $? -eq 0 ]; then
		if [ $ENABLE_DATA_CENTER -eq 1 -a $ENABLE_UPDATE_PROXY -eq 1 ]; then
			echo
			ask_Yorn_question "Do you want to set all connected station to update from this server (proxy role)"
			if [ $? -eq 1  -a  $YES_TO_ALL -eq 0 ]; then
				SERVER_NAME="AVG Admin Server UpdateProxy"
				OTHER_NAMES="update primary server;update backup server"
				SERVER_URL="+http://$(ifconfig | grep -m 2 'inet ' | grep -v 127 | cut -d: -f2 | awk '{ print $1}'):4158/avgupdate/"
				OTHER_URLS="+http://update.avg.com/softw/12/update/;+http://backup.avg.cz/softw/12/update/"
				ask_default_answer_question "Url update address of this server" SERVER_URL
				ask_default_answer_question "Name for this server" SERVER_NAME
				ask_default_answer_question "Other update servers url" OTHER_URLS
				ask_default_answer_question "Other update servers name" OTHER_NAMES
				avgadmsrv --updateProxyURL "$SERVER_NAME;$OTHER_NAMES;;$SERVER_URL;$OTHER_URLS;"
			fi
		fi
	else
		YES_TO_ALL=0
		echo
		ask_option_question "Attempt to $SELECTED_DB_ACTION_TEXT failed! What to do now?" "OPTION"\
		"Try again with new database connection settings." "Continue without database (AVG DataCenter)." "Abort configuration."
		case "$OPTION" in
			1)
				setup_db_engine
				setup_dc
				;;
			2)
				RESULT=1
				RESULT_TEXT="ERROR: Attempt to $SELECTED_DB_ACTION_TEXT failed! If you want to use AVG DataCenter, please do so manually (see INSTALL, FAQ)."
				;;
			*)
				abort $ABORT_TEXT
				;;
		esac
	fi
}

setup_admsrv()
{
	# Set default values (prior to old config file)
	read_from_confi "$CFG_FILE_PATH" ENABLE_DATA_CENTER "EnableDataCenter" "1"
	read_from_confi "$CFG_FILE_PATH" ENABLE_UPDATE_PROXY "EnableUpdateProxy" "1"
	read_from_confi "$CFG_FILE_PATH" DB_TYPE "DatabaseType" "Firebird"
	read_from_confi "$CFG_FILE_PATH" DBA_USER_NAME "DBAUserName" "SYSDBA"
	read_from_confi "$CFG_FILE_PATH" DBA_USER_PW "DBAPassword" "masterkey"
	read_from_confi "$DBS_FILE_PATH" NEW_DB_VERSION '<DbVersion' "" "=#/"

	# Choose at least one server role, or abort setup (with quiet uninstall)
	ask_Yorn_question "Set DataCenter role"; ENABLE_DATA_CENTER=$?
	ask_Yorn_question "Set UpdateProxy role"; ENABLE_UPDATE_PROXY=$?
	if [ $ENABLE_DATA_CENTER -eq 0  -a  $ENABLE_UPDATE_PROXY -eq 0 ] ; then
		echo "Please choose at least one server role."
		ask_Yorn_question "Continue installation."
		[ $? -eq 1 ] && setup_admsrv || abort $ABORT_TEXT
	fi
	write_to_confi "$CFG_FILE_PATH" "EnableDataCenter" "$ENABLE_DATA_CENTER"
	write_to_confi "$CFG_FILE_PATH" "EnableUpdateProxy" "$ENABLE_UPDATE_PROXY"

	if [ $ENABLE_DATA_CENTER -eq 1 ]; then
		setup_db_engine
		setup_dc
	fi
}


########
# START
#

# Include script files
include "${0%/*}/../etc/init.d/avgdinit.conf"
include "$DST_TREE/etc/init.d/functions.setup"

# Values exported from install.sh
YES_TO_ALL=${YES_TO_ALL:-0}
RESULT=0
RESULT_TEXT="AVG Admin Server for Linux successfully configured."
ABORT_TEXT="Setup aborted."

# Paths for files and directories
DBA_FILE_PATH="$DST_TREE/var/db/avgdb.fdb"
DBS_FILE_PATH="$DST_TREE/var/db/scripts/FireBird_Scripts/firebird.sql"
CFG_FILE_PATH="$DST_TREE/etc/avgadmsrv.conf"
DIAG_FILE_PATH="${DST_TREE}/etc/init.d/functions.diag"
QUESTIONS_ALIGN=50

# Firebird specific settings
Firebird_SERVER_VERSION_MIN=2
Firebird_SERVER_VERSION_MIN_MINOR=1
Firebird_SERVER_PORT_DEFAULT="3050"

# Code for directly launched script
process_command_line $@

# Check privileges
if [ "$(id -u)" -ne 0 ]; then
	abort "Try to run this as root."
fi

# check running AVG Admin Server
if [ -n "$(ps axo comm | grep ^avgadmsrv$)" ]; then
	abort "AVG Admin Server is running. Stop it first, please."
fi

# Print head text
clear
echo "======================================"
echo "   AVG Admin Server for Linux setup   "
echo "======================================"

setup_admsrv

echo
echo $RESULT_TEXT
echo
exit $RESULT
